#! /bin/sh
#
# Copyright by The HDF Group.
# All rights reserved.
#
# This file is part of the HDF NPP Aggregation Tool (nagg). The full
# NPP Aggregation Tool copyright notice, including terms governing use,
# modification, and redistribution, is contained in the file COPYING,
# which can be found at the root of the source code distribution tree.
# If you do not have access to this file, you may request a copy from
# help@hdfgroup.org.
#
#
# More tests for the HDF NPP aggregation tool
# These are additional tests that require more reosurces such as:
# larger or more input data files;
# more disk space;
# longer execution time.
# The tests will run if test data directory is avaiable and $NAGG_TEST_MORE
# is defined.


# The build (current) directory might be different than the source directory.
if test -z "$srcdir"; then
    srcdir=.
fi

# Include Nagg test functions.
. $srcdir/nagg_functions.sh

# Main body.
dir=$SRC_TOOL_TESTFILES/extensive_data

# Continue only if $NAGG_TESST_MORE is defined and test data directory is
# available.
if [ -z "$NAGG_TESST_MORE" ]; then
    echo "\$NAGG_TESST_MORE is not defined."
    echo "Tests skipped."
    exit $EXIT_SUCCESS
fi
if [ ! -d $dir ]; then
    echo "Test data directory ($dir) is not available."
    echo "Tests skipped."
    exit $EXIT_SUCCESS
fi

#====================================================
# Tests of intermediate products.  These don't take very long to run,
# but some of the files are very large, so the files were put in
# /mnt/hdf/JPSS/CLASS/GRAVITE instead of in test/testfiles.
#====================================================
#test packaging and aggregating 2 granules of many products
nfiles=2
dfiles=$dir/GRAVITE/GAERO/VA*.h5
echo ============================================
echo This takes $nfiles input files
echo ============================================
nx=`echo $dfiles | wc -w`
if [ $nx -ne $nfiles ]; then
    echo "number of input files ($nx) is not as expected ($nfiles)."
    nerrors=`expr $nerrors + 1`
else
    CHECK_OUTPUT gaero_products.txt -n2 -tVAOOO $dfiles
fi

nfiles=16
dfiles="$dir/GRAVITE/GCLDO/IVC*.h5 $dir/GRAVITE/GCLDO/VC*.h5"
echo ============================================
echo This takes $nfiles input files
echo ============================================
nx=`echo $dfiles | wc -w`
if [ $nx -ne $nfiles ]; then
    echo "number of input files ($nx) is not as expected ($nfiles)."
    nerrors=`expr $nerrors + 1`
else
    CHECK_OUTPUT gcldo_products.txt -n2 -tIVCLT,VCBHO,VCCLO,VCEPO,VCOTO,VCTHO,VCTPO,VCTTO $dfiles
fi

nfiles=14
dfiles="$dir/GRAVITE/GITCO/IV*.h5 $dir/GRAVITE/GITCO/V*.h5"
echo ============================================
echo This takes $nfiles input files
echo ============================================
nx=`echo $dfiles | wc -w`
if [ $nx -ne $nfiles ]; then
    echo "number of input files ($nx) is not as expected ($nfiles)."
    nerrors=`expr $nerrors + 1`
else
    CHECK_OUTPUT gitco_products.txt -n 2 -t IVIIC,IVIIW,IVIQF,IVIRT,IVSTP,VIVIO,VSCMO $dfiles
fi

nfiles=52
dfiles="$dir/GRAVITE/GMODO/II*.h5 $dir/GRAVITE/GMODO/IV*.h5 $dir/GRAVITE/GMODO/SVM*.h5"
echo ============================================
echo This takes $nfiles input files
echo ============================================
nx=`echo $dfiles | wc -w`
if [ $nx -ne $nfiles ]; then
    echo "number of input files ($nx) is not as expected ($nfiles)."
    nerrors=`expr $nerrors + 1`
else
    CHECK_OUTPUT gmodo_products.txt -n 2 -t IICMO,IVBPX,IVCBH,IVCOP,IVCTP,IVICC,IVIWT,IVPCM,IVPCP,IVPTP,SVM01,SVM02,SVM03,SVM04,SVM05,SVM06,SVM07,SVM08,SVM09,SVM10,SVM11,SVM12,SVM13,SVM14,SVM15,SVM16 $dfiles
fi

nfiles=26
dfiles="$dir/GRAVITE/GMTCO/IV*.h5 $dir/GRAVITE/GMTCO/V*.h5"
echo ============================================
echo This takes $nfiles input files
echo ============================================
nx=`echo $dfiles | wc -w`
if [ $nx -ne $nfiles ]; then
    echo "number of input files ($nx) is not as expected ($nfiles)."
    nerrors=`expr $nerrors + 1`
else
    CHECK_OUTPUT gmtco_products.txt -n 2 -t IVAMI,IVAOT,IVISR,IVSIC,VISAO,VISTO,VLSTO,VOCCO,VSCDO,VSICO,VSSTO,VSTYO,VSUMO $dfiles
fi

nfiles=6
dfiles="$dir/GRAVITE/GONPO/I*.h5 $dir/GRAVITE/GONPO/S*.h5"
echo ============================================
echo This takes $nfiles input files
echo ============================================
nx=`echo $dfiles | wc -w`
if [ $nx -ne $nfiles ]; then
    echo "number of input files ($nx) is not as expected ($nfiles)."
    nerrors=`expr $nerrors + 1`
else
    CHECK_OUTPUT gonpo_products.txt -n 2 -t IMOPO,INPAK,SOMPS $dfiles
fi

nfiles=2
dfiles=$dir/GRAVITE/GOTCO/IN*.h5
echo ============================================
echo This takes $nfiles input files
echo ============================================
nx=`echo $dfiles | wc -w`
if [ $nx -ne $nfiles ]; then
    echo "number of input files ($nx) is not as expected ($nfiles)."
    nerrors=`expr $nerrors + 1`
else
    CHECK_OUTPUT gotco_products.txt -n 2 -t INCTO $dfiles
fi

nfiles=2
dfiles=$dir/GRAVITE/ICDBG/IV*.h5
echo ============================================
echo This takes $nfiles input files
echo ============================================
nx=`echo $dfiles | wc -w`
if [ $nx -ne $nfiles ]; then
    echo "number of input files ($nx) is not as expected ($nfiles)."
    nerrors=`expr $nerrors + 1`
else
    CHECK_OUTPUT icdbg_products.txt -n 2 -t IVCDB $dfiles
fi

# ===================================================
# The following tests involve many input files and takes a long time to
# run.  Verify the number of input files before launching the long running
# tests.
# ===================================================
#
# Test many input files for one product, SVM01.
# Verify number of input files
nfiles=120
dfiles=$dir/122293334/SVM01*.h5
echo ============================================
echo This takes $nfiles input files
echo ============================================
nx=`echo $dfiles | wc -w`
if [ $nx -ne $nfiles ]; then
    echo "number of input files ($nx) is not as expected ($nfiles)."
    nerrors=`expr $nerrors + 1`
else
    CHECK_OUTPUT manyinputfilesSVM01.txt -n 16 -t SVM01 $dfiles
fi

# Test several input files for compatible products SVM01-SVM16 and IICMO
# with their geolocation product GMODO.  The granule duration for IICMO
# was originally incorrect which caused the expected ouput of four granules
# per each product in one file to be spread over 3 files instead.
# Also, Paul Martinolich, who provided the files, discovered that if -g no
# is used and GMODO is added to the product list, the geolocation files can
# be added to the data product files at a later time.  Since we only have
# one designated GEO product for each data product, but there are apparently
# multiple valid options (GMTCO) this behavior is desirable.  If this test
# fails because the unintended behavior is somehow disallowed, another
# option should be provided.
#
# Verify number of input files
nfiles=72
dfiles=$dir/PaulM-2012-07-30/*.h5
echo ============================================
echo This takes $nfiles input files
echo ============================================
nx=`echo $dfiles | wc -w`
if [ $nx -ne $nfiles ]; then
    echo "number of input files ($nx) is not as expected ($nfiles)."
    nerrors=`expr $nerrors + 1`
else
    CHECK_FAIL -n 4 -g no -t SVM01,SVM02,SVM03,SVM04,SVM05,SVM06,SVM07,SVM08,SVM09,SVM10,SVM11,SVM12,SVM13,SVM14,SVM15,SVM16,IICMO,GMODO $dfiles
fi

# The above command will now fail.  The replacement option provided is tested here.
nfiles=72
dfiles=$dir/PaulM-2012-07-30/*.h5
echo ============================================
echo This takes $nfiles input files
echo ============================================
nx=`echo $dfiles | wc -w`
if [ $nx -ne $nfiles ]; then
    echo "number of input files ($nx) is not as expected ($nfiles)."
    nerrors=`expr $nerrors + 1`
else
    CHECK_OUTPUT inputfilesSVM01-16-IICMO-GMODO.txt -n 4 -t SVM01,SVM02,SVM03,SVM04,SVM05,SVM06,SVM07,SVM08,SVM09,SVM10,SVM11,SVM12,SVM13,SVM14,SVM15,SVM16,IICMO -g GMODO $dfiles
fi

# Test many many input files for products, SVM01-SVM16
# put the output in tmptest-out for the large file deaggregation test that follows
outdir="tmptest-out"
test -e $outdir || mkdir $outdir
nfiles=1885
dfiles=$dir/122293334/SVM*.h5
echo ============================================
echo This takes $nfiles input files
echo ============================================
nx=`echo $dfiles | wc -w`
if [ $nx -ne $nfiles ]; then
    echo "number of input files ($nx) is not as expected ($nfiles)."
    nerrors=`expr $nerrors + 1`
else
#    CHECK_OUTPUT manyinputfilesSVM01-16.txt -n 16 -t SVM01,SVM02,SVM03,SVM04,SVM05,SVM06,SVM07,SVM08,SVM09,SVM10,SVM11,SVM12,SVM13,SVM14,SVM15,SVM16 $dfiles
#replacing test above for a larger granules_selected limit.
    CHECK_OUTPUT manyinputfilesSVM01-16_129.txt -n 129 -d $outdir -t SVM01,SVM02,SVM03,SVM04,SVM05,SVM06,SVM07,SVM08,SVM09,SVM10,SVM11,SVM12,SVM13,SVM14,SVM15,SVM16 $dfiles

fi

# de-aggregate the output files of the previous test
dfiles=$outdir/GMODO-SVM*.h5
CHECK_OUTPUT manyoutputfilesSVM01-16.txt -t SVM01,SVM02,SVM03,SVM04,SVM05,SVM06,SVM07,SVM08,SVM09,SVM10,SVM11,SVM12,SVM13,SVM14,SVM15,SVM16 -S $dfiles
rm -rf $outdir


#Test --onefile option with and without --nofill
dfile=$dir/GMODO-SVM01/GMODO-SVM01_npp_d20130506*.h5
CHECK_OUTPUT svm01-onefile-nofill.txt --onefile --nofill -t SVM01 $dfile
CHECK_OUTPUT svm01-onefile.txt --onefile -t SVM01 $dfile
# This one should advance both products together with a real granule thrown in for the second product part way through.
#src/nagg --onefile -t SVM01,SVM05 /mnt/scr1/JPSS/CLASS/122293334/SVM0[15]*_npp_d20120423_t2359080_e0000322_b02534_*_noaa_ops.h5 /mnt/scr1/JPSS/CLASS/122293334/SVM05_npp_d20120424_t0003243_e0004484_b02534_*_noaa_ops.h5 /mnt/scr1/JPSS/CLASS/122293334/SVM0[15]*_npp_d20120424_t0010313_e0011555_b02534_*_noaa_ops.h5

# Post results and exit.
echo
if test $nerrors -eq 0 ; then
    echo "All $TESTNAME tests passed."
    exit $EXIT_SUCCESS
else
    echo "$TESTNAME tests failed with $nerrors errors."
    exit $EXIT_FAILURE
fi
